<cfsilent>
<!---
CF_FORMAT
by Keith Gaughan <kmgaughan@eircom.net>

StructuredText Formatting Engine with support for Wikiwords.

Copyright (c) Keith Gaughan, 2003.
This software is free; you can redistribute it and/or modify it
under the terms of the Design Science License (DSL). If you didn't
receive a copy of the DSL with this software, one can be obtained
at http://www.dsl.org/copyleft/dsl.txt.
--->

<!--- Protect from independent invocation. --->
<cfif GetCurrentTemplatePath() eq GetBaseTemplatePath()><cfabort></cfif>

<cfif IsDefined("ATTRIBUTES.variable")>
	<cfset body = Evaluate("CALLER." & ATTRIBUTES.variable)>
<cfelseif IsDefined("ATTRIBUTES.text")>
	<cfset body = ATTRIBUTES.text>
<cfelse>
    <cfthrow message="Either of attributes &lsquo;variable&rsquo; or &lsquo;text&rsquo; must be defined.">
</cfif>

<cfparam name="ATTRIBUTES.imagesPath" default="./images/" type="string">

</cfsilent><cfscript>
function ProcessBlocks(str, fn, startTok, endTok)
{
    var out = "";
    var blk = "";
    var iBegin = 1;
    var iEnd = 0;

    for (;;)
    {
        // Find everything up as far as what we can replace.
        iEnd = Find(endTok, str, iBegin);
        if (iEnd EQ 0)
           iEnd = Len(str);
        blk = Mid(str, iBegin, iEnd - iBegin + 1);
        out = out & fn(blk);
        iBegin = iEnd + 1;
        if (iEnd EQ Len(str))
            return out;
        iEnd = Find(startTok, str, iBegin);
        if (iEnd EQ 0)
            iEnd = Len(tmp);
        out = out & Mid(str, iBegin, iEnd - iBegin + 1);
        iBegin = iEnd + 1;
    }
}

function MarkExternalLinks(blk)
{
    return REReplace(blk, "((ht|f)tp://[-A-Za-z0-9_.]+/[-A-Za-z0-9_./%?&;=##~]*)",
        """\1""=""\1""", "ALL");
}

function ApplyMainMarkup(blk)
{
    // Emphasis
    blk = REReplace(blk, "\*\*([^ ][^*]*)\*\*", "<strong>\1</strong>", "ALL");
    blk = REReplace(blk, "\*([^ ][^*]*)\*", "<em>\1</em>", "all");

    // Smilies
    blk = Replace(blk, ":-)",
        "<img src=""#ATTRIBUTES.imagesPath#smilie.png"" width=""12"" height=""12"" alt="":-)"" />", "ALL");

    return blk;
}

function Wikify(blk)
{
	var iStart  = 1;
	var matches = "";
	var pos     = "";
	var len     = "";
	var prefix  = "";
	var word    = "";
	var link    = "";
	var lft     = "";
	var rgt     = "";

	for (;;)
	{
	    // The technically correct regex for matching the {{..}} words is
	    // \[\[([^\]]+(][^\]]+)*)]] However, for some reason, this does not
		// work properly, so I'm using the following buggy but working one:

		matches = REFind("([^A-Za-z])([A-Z][a-z]*[A-Z][A-Za-z]*)", blk, iStart, "Yes");
		pos = matches.pos;
		len = matches.len;

		if (pos[1] EQ 0)
			return blk;

		prefix = Mid(blk, pos[2], len[2]);
		word   = Mid(blk, pos[3], len[3]);

		if (StructKeyExists(ATTRIBUTES.wikiWords, REReplace(word, "[^[:alpha:]-]", "", "ALL")))
			link = "<a href=""./?doc=#word#"">#word#</a>";
		else
			link = "#word#<a href=""./?doc=#word#"">?</a>";

		lft = "";
		if (pos[1] - 1 GT 0)
			lft = Left(blk, pos[1] - 1);
		rgt = "";
		if (Len(blk) - pos[1] - len[1] + 1 GT 0)
			rgt = Right(blk, Len(blk) - pos[1] - len[1] + 1);
		blk = "#lft##prefix##link##rgt#";
		iStart = pos[1] + Len(link);
	}
}

function ApplyNiceSymbols(blk)
{
    // Add nice symbols.
    blk = Replace(blk, "``", "&ldquo;", "all");
    blk = Replace(blk, "''", "&rdquo;", "all");
    blk = Replace(blk, "`", "&lsquo;", "all");
    blk = Replace(blk, "'", "&rsquo;", "all");
    blk = Replace(blk, "(c)", "&copy;", "all");
    blk = Replace(blk, "&lt;&lt;", "&laquo;", "all");
    blk = Replace(blk, "&gt;&gt;", "&raquo;", "all");

	return blk;
}

function KillExtraQuotes(blk)
{
    // Remove any non-paired doubleprimes.
    blk = Replace(blk, """""", "&quot;", "ALL");
    blk = Replace(blk, """", "", "ALL");
	return blk;
}

/////////////////////////////////////////////////////////////////////////////

// Kill off all the nasty characters.
body = HTMLEditFormat(StripCR(body));

body = "<p>" & Replace(body, "&quot;", """", "all") & "</p>";

// Paragraphs and horizontal rules
body = Replace(body, Chr(10) & "----",
    Chr(10) & "</p><hr /><p>", "all");
body = Replace(body, "--", "&mdash;", "all");
body = Replace(body, Chr(10) & Chr(10),
    "</p>" & Chr(10) & Chr(10) & "<p>", "all");
body = Replace(body, "<p></p>", "", "all");

// Ordered and unordered lists
body = Replace(body, "<p> * ", "<ul>" & Chr(10) & "<li>", "all");
body = Replace(body, Chr(10) & " * ", "</li>" & Chr(10) & "<li>", "all");
body = REReplace(body, "<li>([^<]*(</[^p][^>][^<]*)*)</p>",
    "<li>\1</li>" & Chr(10) & "</ul>", "all");
body = Replace(body, "<p> ## ", "<ol>" & Chr(10) & "<li>", "all");
body = Replace(body, Chr(10) & " ## ", "</li>" & Chr(10) & "<li>", "all");
body = REReplace(body, "<li>([^<]*(</[^p][^>][^<]*)*)</p>",
    "<li>\1</li>" & Chr(10) & "</ol>", "all");

body = ProcessBlocks(body, MarkExternalLinks, """", """");
body = ProcessBlocks(body, ApplyMainMarkup,   """","""");
if (IsDefined("ATTRIBUTES.wikiwords"))
	body = ProcessBlocks(body, Wikify,        """","""");
body = ProcessBlocks(body, ApplyNiceSymbols,  """","""");

// External links & Email addresses
if (IsDefined("ATTRIBUTES.wikiwords"))
{
	body = REReplace(body, """([^""]+)""=""([^""]+)""",
	    "<a href=""\2"">\1</a>" &
	    "<img src=""./images/external.png"" width=""12"" height=""10"" " &
	    "alt=""[External Link]"" style=""padding:0 2px"">", "all");
}
else
{
	body = REReplace(body, """([^""]+)""=""([^""]+)""",
	    "<a href=""\2"">\1</a>", "all");
}
body = REReplace(body,
    "mailto:([A-Za-z0-9][-A-Za-z0-9._]*@[-A-Za-z0-9._]+\.[A-Za-z]+)",
    "<a href=""mailto:\1"">\1</a>", "all");

body = ProcessBlocks(body, KillExtraQuotes, ">","<");

if (IsDefined("ATTRIBUTES.variable"))
	SetVariable("CALLER." & ATTRIBUTES.variable, body);
else
	WriteOutput(body);
</cfscript>